@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_getaddrinfo.h,v 1.1 2004/05/28 22:17:04 evertonm Exp $
+  $Id: ruli_getaddrinfo.h,v 1.2 2004/06/05 19:42:31 evertonm Exp $
   */
 
 
@@ -27,19 +27,6 @@ Boston, MA 02111-1307, USA.
 #define RULI_GETADDRINFO
 
 
-#ifndef __USE_GNU
-#define __USE_GNU
-#endif
-
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
-
-#ifndef __USE_POSIX
-#define __USE_POSIX
-#endif
-
-
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netdb.h>
@@ -48,6 +35,8 @@ Boston, MA 02111-1307, USA.
 int ruli_getaddrinfo(const char *node, const char *service,
 		     const struct addrinfo *hints, struct addrinfo **_res);
 
+void ruli_freeaddrinfo(struct addrinfo *res);
+
 
 #endif
 
