@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_res.c,v 1.30 2004/06/06 04:27:48 evertonm Exp $
+  $Id: ruli_res.c,v 1.33 2004/06/17 07:21:01 evertonm Exp $
   */
 
 
@@ -30,6 +30,7 @@ Boston, MA 02111-1307, USA.
 
 #include <stdio.h>    /* FIXME: remove me [used for fprintf() debug] */
 
+#include <strings.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -71,6 +72,7 @@ static const ruli_qtype_pair_t ruli_qtype_map[] =
   { "minfo", RULI_RR_TYPE_MINFO },
   { "mx",    RULI_RR_TYPE_MX },
   { "txt",   RULI_RR_TYPE_TXT },
+  { "aaaa",  RULI_RR_TYPE_AAAA },
   { "srv",   RULI_RR_TYPE_SRV },
   { "axfr",  RULI_RR_TYPE_AXFR },
   { "mailb", RULI_RR_TYPE_MAILB },
@@ -93,6 +95,12 @@ static const ruli_qclass_pair_t ruli_qclass_map[] =
 };
 
 
+#ifdef _RULI_SOLARIS_
+/* should be in <strings.h> */
+extern int strcasecmp(const char *s1, const char *s2);
+#endif
+
+
 int ruli_get_qtype_code(const char *qt_name) 
 {
   int i;
@@ -350,7 +358,7 @@ static int get_udp_socket(ruli_res_query_t *res_qry)
 {
   ruli_addr_t *addr = ruli_res_get_curr_serv_addr(res_qry);
   ruli_res_t *res_ctx = res_qry->resolver;
-  int sd;
+  int sd = -1; /* picky compilers */
 
   switch (addr->addr_family) {
   case PF_INET:
@@ -422,10 +430,10 @@ static int start_query(ruli_res_query_t *res_qry)
   /*
    * Set handler for events on socket
    *
-   * fsm will return in fsm_query_done()
+   * fsm will return in _ruli_fsm_query_done()
    */
 
-  query_want_write_udp(res_qry);
+  _ruli_query_want_write_udp(res_qry);
 
   return RULI_RES_OK;
 }
@@ -532,7 +540,7 @@ void ruli_res_query_delete(ruli_res_query_t *res_qry)
   /*
    * Perform state-specific clean-up
    */
-  query_status_done(res_qry);
+  _ruli_query_status_done(res_qry);
    
   /*
    * If we have an answer buffer allocated, release it.
@@ -597,7 +605,7 @@ int ruli_res_get_curr_serv_port(ruli_res_query_t *res_qry)
   return ruli_res_get_curr_server(res_qry)->port;
 }
 
-int get_curr_tcp_socket(ruli_res_query_t *qry) 
+int _ruli_get_curr_tcp_socket(ruli_res_query_t *qry) 
 {
   ruli_res_t    *res_ctx = qry->resolver;
   int           i        = ruli_res_get_curr_server_index(qry);
@@ -710,7 +718,7 @@ static void *do_user_callback(ruli_res_query_t *qry)
 
   fsm returns here
  */
-void *fsm_query_done(ruli_res_query_t *qry)
+void *_ruli_fsm_query_done(ruli_res_query_t *qry)
 {
 #ifdef RULI_RES_DEBUG
   fprintf(stderr, 
