@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_smtp.c,v 1.8 2004/06/06 04:27:48 evertonm Exp $
+  $Id: ruli_smtp.c,v 1.12 2004/06/17 07:28:45 evertonm Exp $
   */
 
 
@@ -205,7 +205,7 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 	  ruli_mx_rdata_t  mx_rdata;
 	  ruli_list_t      *ad_list     = &parse.additional_list;
 	  int              ad_list_size = ruli_list_size(ad_list);
-	  ruli_srv_entry_t *srv_entry   = 0;
+	  ruli_srv_entry_t *srv_entry;
 
 	  if (ruli_parse_rr_mx(&mx_rdata, rr->rdata, rr->rdlength,
 			       (ruli_uint8_t *) qry->answer_buf,
@@ -214,6 +214,21 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 	    return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_PARSE);
 	  }
 
+	  /* 
+	   * Create SRV record and append it to list 
+	   */
+	  srv_entry =_ruli_srv_list_new_entry(&srv_qry->answer_srv_list,
+					      (const char *) \
+					      mx_rdata.target,
+					      mx_rdata.target_len,
+					      mx_rdata.preference,
+					      -1,
+					      srv_qry->srv_fallback_port);
+	  if (!srv_entry) {
+	    ruli_parse_delete(&parse);
+	    return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
+	  }
+
 	  /*
 	   * Search MX target addresses in additional section
 	   */
@@ -242,43 +257,6 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 	     * Save MX target address
 	     */
 
-	    if (!srv_entry) {
-	      
-	      /* Create SRV record */
-	      srv_entry = (ruli_srv_entry_t *) ruli_malloc(sizeof(ruli_srv_entry_t));
-	      if (!srv_entry) {
-		ruli_parse_delete(&parse);
-		return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
-	      }
-	      
-	      /* Create record addr list */
-	      if (ruli_list_new(&srv_entry->addr_list)) {
-		ruli_free(srv_entry);
-		ruli_parse_delete(&parse);
-		return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
-	      }
-	      
-	      /* Save record */
-	      if (ruli_list_push(&srv_qry->answer_srv_list, srv_entry)) {
-		ruli_list_delete(&srv_entry->addr_list);
-		ruli_free(srv_entry);
-		ruli_parse_delete(&parse);
-		return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
-	      }
-	      
-	      /* 
-	       * Init record 
-	       */
-	      srv_entry->priority = mx_rdata.preference;
-	      srv_entry->weight   = -1;
-	      srv_entry->port     = srv_qry->srv_fallback_port;
-	      
-	      assert(srv_qry->srv_domain_len <= RULI_LIMIT_DNAME_ENCODED);
-	      srv_entry->target_len = mx_rdata.target_len;
-	      memcpy(srv_entry->target, mx_rdata.target, 
-		     srv_entry->target_len);
-	    } /* Create SRV record */
-	    
 	    /* Allocate space */
 	    addr = (ruli_addr_t *) ruli_malloc(sizeof(*addr));
 	    if (!addr) {
@@ -311,8 +289,43 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 	assert(curr_srv_list_size >= prev_srv_list_size);
 	
 	if (curr_srv_list_size > prev_srv_list_size) {
+#ifdef RULI_SMTP_DEBUG
+	  {
+	    int ii;
+	    ruli_list_t *list = &srv_qry->answer_srv_list;
+	    
+	    fflush(stdout);
+	    for (ii = 0; ii < ruli_list_size(list); ++ii) {
+	      ruli_srv_entry_t *srv_entry = \
+		(ruli_srv_entry_t *) ruli_list_get(list, ii);
+	      
+	      fprintf(stderr,
+		      "DEBUG: %s: %s(): answer SRV RR: "
+		      "priority=%d weight=%d port=%d\n",
+		      __FILE__, __PRETTY_FUNCTION__,
+		      srv_entry->priority, srv_entry->weight, srv_entry->port);
+	    }
+	    fflush(stderr);
+	  }
+#endif
+
 	  ruli_parse_delete(&parse);
-	  return fall_query_done(srv_qry, RULI_SRV_CODE_OK);
+
+	  /* Dispose current IN MX fallback query */
+	  f_query_done(srv_qry);
+
+	  /*
+	   * Launch queries to fill missing addresses, if any
+	   */
+
+	  assert(!srv_qry->walk_query);
+	  assert(srv_qry->walk_index == -1);
+	  assert(srv_qry->walk_qtype == -1);
+	  
+	  srv_qry->walk_index = 0;
+	  srv_qry->walk_qtype = _ruli_reset_qtype(srv_qry->srv_options);
+	  
+	  return _ruli_srv_answer_walk(srv_qry);
 	}
       }
 
@@ -324,17 +337,18 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 	      qry->query_id, qry->answer_code, qry->answer_header.rcode);
 #endif
 
-    } /* MX search */    
+    } /* MX scanning loop */    
 
     ruli_parse_delete(&parse);
     
   } /* parsed answer context */
   
-  /* Dispose failed IN MX fallback query */
+  /* Dispose failed, current IN MX fallback query,
+   that is replaced by IN A fallback query below */
   f_query_done(srv_qry);
 
   /* Try default IN A fallback query */
-  return _srv_answer_fallback_addr(srv_qry);
+  return _ruli_srv_answer_fallback_addr(srv_qry);
 }
 
 /*
