@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_srv.c,v 1.61 2004/06/09 17:57:15 evertonm Exp $
+  $Id: ruli_srv.c,v 1.68 2004/06/17 14:41:05 evertonm Exp $
   */
 
 
@@ -119,15 +119,13 @@ static void *walk_query_done(ruli_srv_t *srv_qry, int srv_result_code)
   return query_done(srv_qry, srv_result_code);
 }
 
-static void *srv_answer_walk(ruli_srv_t *srv_qry);
-
 static int need_query_in_a(int qtype, long options)
 {
   return ( (qtype == RULI_RR_TYPE_AAAA) &&
 	   !(options & RULI_RES_OPT_SRV_NOINET) );
 }
 
-static int reset_qtype(long options)
+int _ruli_reset_qtype(long options)
 {
   /* can't disable all address families */
   assert( !((options & RULI_RES_OPT_SRV_NOINET) &&
@@ -205,7 +203,7 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 
 	srv_qry->walk_qtype = RULI_RR_TYPE_A;
 
-	return srv_answer_walk(srv_qry); /* repeat walk index */
+	return _ruli_srv_answer_walk(srv_qry); /* repeat walk index */
       }
 
       return walk_query_done(srv_qry, RULI_SRV_CODE_WALK_RCODE);
@@ -320,9 +318,12 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 	assert(!result);
 	
 	fprintf(stderr, 
-		"DEBUG: on_walk_answer(): target=%s len=%d w_qtype=%d w_index=%d: "
+		"DEBUG: %s: %s(): "
+		"target=%s len=%d w_qtype=%d w_index=%d: "
 		"BUT: no matching address record\n", 
-		target_txt, target_txt_len, srv_qry->walk_qtype, srv_qry->walk_index);
+		__FILE__, __PRETTY_FUNCTION__,
+		target_txt, target_txt_len, 
+		srv_qry->walk_qtype, srv_qry->walk_index);
       }
 #endif
 
@@ -333,7 +334,7 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 
 	srv_qry->walk_qtype = RULI_RR_TYPE_A;
 
-	return srv_answer_walk(srv_qry); /* repeat walk index */
+	return _ruli_srv_answer_walk(srv_qry); /* repeat walk index */
       }
 
       return walk_query_done(srv_qry, RULI_SRV_CODE_WALK_EMPTY);
@@ -347,7 +348,7 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 
     srv_qry->walk_qtype = RULI_RR_TYPE_A;
 
-    return srv_answer_walk(srv_qry); /* repeat walk index */
+    return _ruli_srv_answer_walk(srv_qry); /* repeat walk index */
   }
 
   /* Dispose auxiliary query as it will be re-created
@@ -355,12 +356,12 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
   w_query_done(srv_qry);
 
   /* For the next target, use the first address type */
-  srv_qry->walk_qtype = reset_qtype(srv_qry->srv_options);
+  srv_qry->walk_qtype = _ruli_reset_qtype(srv_qry->srv_options);
 
   /* In future walk query, consider next answer target */
   ++srv_qry->walk_index;
 
-  return srv_answer_walk(srv_qry);
+  return _ruli_srv_answer_walk(srv_qry);
 }
 
 static int find_addr_type(const ruli_list_t *addr_list, int qtype)
@@ -372,7 +373,7 @@ static int find_addr_type(const ruli_list_t *addr_list, int qtype)
   case RULI_RR_TYPE_A:
 
     for (i = 0; i < list_size; ++i) {
-      ruli_addr_t *addr = ruli_list_get(addr_list, i);
+      ruli_addr_t *addr = (ruli_addr_t *) ruli_list_get(addr_list, i);
       if (ruli_addr_family(addr) == PF_INET)
 	return -1;
     }
@@ -381,7 +382,7 @@ static int find_addr_type(const ruli_list_t *addr_list, int qtype)
   case RULI_RR_TYPE_AAAA:
 
     for (i = 0; i < list_size; ++i) {
-      ruli_addr_t *addr = ruli_list_get(addr_list, i);
+      ruli_addr_t *addr = (ruli_addr_t *) ruli_list_get(addr_list, i);
       if (ruli_addr_family(addr) == PF_INET6)
 	return -1;
     }
@@ -400,7 +401,7 @@ static int find_addr_type(const ruli_list_t *addr_list, int qtype)
   We make serialized queries so we don't need
   to keep more than one query reference (walk_query).
  */
-static void *srv_answer_walk(ruli_srv_t *srv_qry)
+void *_ruli_srv_answer_walk(ruli_srv_t *srv_qry)
 {
   ruli_list_t *srv_list     = &srv_qry->answer_srv_list;
   int         srv_list_size = ruli_list_size(srv_list);
@@ -429,7 +430,7 @@ static void *srv_answer_walk(ruli_srv_t *srv_qry)
       assert(!result);
       
       fprintf(stderr, 
-	      "DEBUG: srv_answer_walk(): missing target=%s walk_index=%d walk_qtype=%d\n", 
+	      "DEBUG: _ruli_srv_answer_walk(): missing target=%s walk_index=%d walk_qtype=%d\n", 
 	      target_txt, srv_qry->walk_index, srv_qry->walk_qtype);
     }
 #endif
@@ -438,7 +439,8 @@ static void *srv_answer_walk(ruli_srv_t *srv_qry)
      * Allocate space for auxiliary walk query
      */
     assert(!srv_qry->walk_query);
-    srv_qry->walk_query = (ruli_res_query_t *) ruli_malloc(sizeof(ruli_res_query_t));
+    srv_qry->walk_query = \
+      (ruli_res_query_t *) ruli_malloc(sizeof(ruli_res_query_t));
     if (!srv_qry->walk_query)
       return query_done(srv_qry, RULI_SRV_CODE_WALK_OTHER);
 
@@ -612,7 +614,8 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
     for (;;) {
       ruli_list_t      *an_list     = &parse.answer_list;
       int              an_list_size = ruli_list_size(an_list);
-      ruli_srv_entry_t *srv_entry   = prev_srv_list_size ? ruli_list_top(&srv_qry->answer_srv_list) : 0;
+      ruli_srv_entry_t *srv_entry   = prev_srv_list_size ? \
+	(ruli_srv_entry_t *) ruli_list_top(&srv_qry->answer_srv_list) : 0;
       int              i;
 
 #ifdef RULI_SRV_DEBUG
@@ -666,40 +669,21 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 
 	  if (!srv_entry) {
 
-	    /* Create record */
-	    srv_entry = (ruli_srv_entry_t *) ruli_malloc(sizeof(ruli_srv_entry_t));
+	    /* 
+	     * Create SRV record and append it to list 
+	     */
+	    srv_entry =_ruli_srv_list_new_entry(&srv_qry->answer_srv_list,
+						srv_qry->srv_domain,
+						srv_qry->srv_domain_len,
+						-1,
+						-1,
+						srv_qry->srv_fallback_port);
 	    if (!srv_entry) {
 	      ruli_parse_delete(&parse);
 	      return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
 	    }
-
-	    /* Create record addr list */
-	    if (ruli_list_new(&srv_entry->addr_list)) {
-	      ruli_free(srv_entry);
-	      ruli_parse_delete(&parse);
-	      return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
-	    }
-
-	    /* Save record */
-	    if (ruli_list_push(&srv_qry->answer_srv_list, srv_entry)) {
-	      ruli_list_delete(&srv_entry->addr_list);
-	      ruli_free(srv_entry);
-	      ruli_parse_delete(&parse);
-	      return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
-	    }
-
-	    /* 
-	     * Init record 
-	     */
-	    srv_entry->priority = -1;
-	    srv_entry->weight   = -1;
-	    srv_entry->port     = srv_qry->srv_fallback_port;
-
-	    assert(srv_qry->srv_domain_len <= RULI_LIMIT_DNAME_ENCODED);
-	    srv_entry->target_len = srv_qry->srv_domain_len;
-	    memcpy(srv_entry->target, srv_qry->srv_domain, 
-		   srv_entry->target_len);
-	  }
+	    
+	  } /* Create SRV record */
 
 	  /* Allocate space */
 	  addr = (ruli_addr_t *) ruli_malloc(sizeof(*addr));
@@ -856,7 +840,7 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 /*
   Fallback to 'addr'
  */
-void *_srv_answer_fallback_addr(ruli_srv_t *srv_qry)
+void *_ruli_srv_answer_fallback_addr(ruli_srv_t *srv_qry)
 {
 #ifdef RULI_SRV_DEBUG
   {
@@ -885,7 +869,7 @@ void *_srv_answer_fallback_addr(ruli_srv_t *srv_qry)
   if (!srv_qry->fall_query)
     return query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
 
-  srv_qry->fall_qtype = reset_qtype(srv_qry->srv_options);
+  srv_qry->fall_qtype = _ruli_reset_qtype(srv_qry->srv_options);
 
   /*
    * Initialize fallback query arguments
@@ -999,6 +983,17 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
   assert(qry->answer_header.rcode == RULI_RCODE_NOERROR);
   assert(qry->answer_header.ancount > 0);
 
+  /* 
+   * Processing of SRV answer:
+   *
+   * Step 1/6: One SRV RR with target == '.' ?
+   * Step 2/6: Parse rdata portion of all SRV RRs
+   * Step 3/6: Sort SRV RRs by priority
+   * Step 4/6: Select SRV RRs by random weighted order
+   * Step 5/6: Build list of srv answers by inspecting additional section
+   * Step 6/6: Launch queries to fill missing addresses, if any
+   */
+
   /* 
    * Step 1/6: One SRV RR with target == '.' ?
    */
@@ -1111,7 +1106,8 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 
     fflush(stdout);
     for (i = 0; i < ruli_list_size(list); ++i) {
-      ruli_srv_rdata_t *srv_rdata = ruli_list_get(list, i);
+      ruli_srv_rdata_t *srv_rdata = \
+	(ruli_srv_rdata_t *) ruli_list_get(list, i);
 
       fflush(stderr);
       fprintf(stderr,
@@ -1135,7 +1131,8 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
      * Handle every RR based on priority (higher priority first)
      */
     for (j = 0; j < src_list_size; ++j) {
-      ruli_srv_rdata_t *srv_rdata    = (ruli_srv_rdata_t *) ruli_list_get(src_list, j);
+      ruli_srv_rdata_t *srv_rdata    = \
+	(ruli_srv_rdata_t *) ruli_list_get(src_list, j);
       ruli_list_t      *dst_list     = &srv_qry->pri_srv_list;
       int              dst_list_size = ruli_list_size(dst_list);
       int              i;
@@ -1160,7 +1157,8 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 	   * Find begin of next priority and insert there
 	   */
 	  for (; i < dst_list_size; ++i) {
-	    ruli_srv_rdata_t *s_rd = (ruli_srv_rdata_t *) ruli_list_get(dst_list, i);
+	    ruli_srv_rdata_t *s_rd = \
+	      (ruli_srv_rdata_t *) ruli_list_get(dst_list, i);
 
 	    if (srv_rdata->priority != s_rd->priority)
 	      break;
@@ -1193,7 +1191,8 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 
     fflush(stdout);
     for (i = 0; i < ruli_list_size(list); ++i) {
-      ruli_srv_rdata_t *srv_rdata = ruli_list_get(list, i);
+      ruli_srv_rdata_t *srv_rdata = \
+	(ruli_srv_rdata_t *) ruli_list_get(list, i);
 
       fflush(stderr);
       fprintf(stderr,
@@ -1291,7 +1290,8 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 
     fflush(stdout);
     for (i = 0; i < ruli_list_size(list); ++i) {
-      ruli_srv_rdata_t *srv_rdata = ruli_list_get(list, i);
+      ruli_srv_rdata_t *srv_rdata = \
+	(ruli_srv_rdata_t *) ruli_list_get(list, i);
 
       fflush(stderr);
       fprintf(stderr,
@@ -1304,7 +1304,7 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 #endif
 
   /*
-   * Step 5/6: Build list of srv answers
+   * Step 5/6: Build list of srv answers by inspecting additional section
    */
   {
     ruli_list_t *src_list     = &srv_qry->wei_srv_list;
@@ -1314,6 +1314,18 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 
     assert(ruli_list_size(dst_list) == 0);
 
+#ifdef RULI_SRV_DEBUG
+    {
+      fflush(stdout);
+      fprintf(stderr,
+	      "DEBUG: %s: %s(): "
+	      "BEFORE addit inspection: srv_rr_list_size=%d srv_answer_list_size=%d\n",
+	      __FILE__, __PRETTY_FUNCTION__,
+	      src_list_size, ruli_list_size(&srv_qry->answer_srv_list));
+      fflush(stderr);
+    }
+#endif
+
     /*
      * Build answer records inspecting additional section
      */
@@ -1321,39 +1333,18 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
     /* Scan all targets */
     for (i = 0; i < src_list_size; ++i) {
       ruli_srv_rdata_t *rd = (ruli_srv_rdata_t *) ruli_list_get(src_list, i);
+      ruli_srv_entry_t *srv_entry;
 
-      /* Create record */
-      ruli_srv_entry_t *srv_entry = (ruli_srv_entry_t *) ruli_malloc(sizeof(ruli_srv_entry_t));
+      /* Create SRV entry and append it to list */
+      srv_entry =_ruli_srv_list_new_entry(dst_list,
+					  (const char *) rd->target,
+					  rd->target_len,
+					  rd->priority,
+					  rd->weight,
+					  rd->port);
       if (!srv_entry)
 	return query_done(srv_qry, RULI_SRV_CODE_MALLOC);
 
-      /* Create record addr list */
-      if (ruli_list_new(&srv_entry->addr_list)) {
-	ruli_free(srv_entry);
-	return query_done(srv_qry, RULI_SRV_CODE_LIST);
-      }
-
-      /* Save record */
-      if (ruli_list_push(dst_list, srv_entry)) {
-	ruli_list_delete(&srv_entry->addr_list);
-	ruli_free(srv_entry);
-	return query_done(srv_qry, RULI_SRV_CODE_LIST);
-      }
-
-      /*
-       * Init record
-       */
-
-      srv_entry->priority = rd->priority;
-      srv_entry->weight   = rd->weight;
-      srv_entry->port     = rd->port;
-
-      /* As RFC2782 states the SRV target comes uncompressed,
-	 we can just copy it */
-      assert(rd->target_len <= RULI_LIMIT_DNAME_ENCODED);
-      srv_entry->target_len = rd->target_len;
-      memcpy(srv_entry->target, rd->target, rd->target_len);
-
       /*
        * Look up target address(es) in additional section
        */
@@ -1400,8 +1391,21 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 
     } /* for */
 
+#ifdef RULI_SRV_DEBUG
+    {
+      fflush(stdout);
+      fprintf(stderr,
+	      "DEBUG: %s: %s(): "
+	      "AFTER addit inspection: srv_rr_list_size=%d srv_answer_list_size=%d\n",
+	      __FILE__, __PRETTY_FUNCTION__,
+	      src_list_size, ruli_list_size(&srv_qry->answer_srv_list));
+      fflush(stderr);
+    }
+#endif
+
     assert(ruli_list_size(dst_list) == src_list_size);
-  }
+
+  } /* Build list of srv answers by inspecting additional section */
 
 #ifdef RULI_SRV_DEBUG
   {
@@ -1410,15 +1414,15 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 
     fflush(stdout);
     for (i = 0; i < ruli_list_size(list); ++i) {
-      ruli_srv_entry_t *srv_entry = ruli_list_get(list, i);
+      ruli_srv_entry_t *srv_entry = \
+	(ruli_srv_entry_t *) ruli_list_get(list, i);
 
-      fflush(stderr);
       fprintf(stderr,
 	      "DEBUG: on_srv_answer(): answer SRV RR: "
 	      "priority=%d weight=%d port=%d\n",
 	      srv_entry->priority, srv_entry->weight, srv_entry->port);
-      fflush(stderr);
     }
+    fflush(stderr);
   }
 #endif
 
@@ -1431,9 +1435,9 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
   assert(srv_qry->walk_qtype == -1);
 
   srv_qry->walk_index = 0;
-  srv_qry->walk_qtype = reset_qtype(srv_qry->srv_options);
+  srv_qry->walk_qtype = _ruli_reset_qtype(srv_qry->srv_options);
 
-  return srv_answer_walk(srv_qry);
+  return _ruli_srv_answer_walk(srv_qry);
 }
 
 int _ruli_srv_query_submit(ruli_srv_t *srv_qry, 
@@ -1580,7 +1584,7 @@ int ruli_srv_query_submit(ruli_srv_t *srv_qry)
 {
   assert(srv_qry);
 
-  return _ruli_srv_query_submit(srv_qry, _srv_answer_fallback_addr);
+  return _ruli_srv_query_submit(srv_qry, _ruli_srv_answer_fallback_addr);
 }
 
 void ruli_srv_query_delete(ruli_srv_t *srv_qry)
@@ -1640,7 +1644,7 @@ int ruli_srv_rcode(ruli_srv_t *srv_qry)
 {
   ruli_srv_rcode_kind_t kind = ruli_srv_rcode_kind(srv_qry);
 
-  int rcode;
+  int rcode = -1; /* picky compilers */
 
   switch (kind) {
   case RULI_SRV_RCODE_NONE:
@@ -1666,4 +1670,44 @@ int ruli_srv_rcode(ruli_srv_t *srv_qry)
   return rcode;
 }
 
+ruli_srv_entry_t *_ruli_srv_list_new_entry(ruli_list_t *srv_list,
+					   const char *target,
+					   int target_len,
+					   int priority,
+					   int weight,
+					   int port)
+{
+  ruli_srv_entry_t *srv_entry;
 
+  /* Create SRV record */
+  srv_entry = (ruli_srv_entry_t *) ruli_malloc(sizeof(*srv_entry));
+  if (!srv_entry)
+    return 0;
+
+  /* Create record addr list */
+  if (ruli_list_new(&srv_entry->addr_list)) {
+    ruli_free(srv_entry);
+    return 0;
+  }
+
+  /* Save record */
+  if (ruli_list_push(srv_list, srv_entry)) {
+    ruli_list_delete(&srv_entry->addr_list);
+    ruli_free(srv_entry);
+    return 0;
+  }
+
+  /*
+   * Init record
+   */
+  srv_entry->priority = priority;
+  srv_entry->weight   = weight;
+  srv_entry->port     = port;
+
+  assert(target_len <= RULI_LIMIT_DNAME_ENCODED);
+
+  srv_entry->target_len = target_len;
+  memcpy(srv_entry->target, target, target_len);
+
+  return srv_entry;
+}
