@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_util.h,v 1.3 2003/01/24 14:39:05 evertonm Exp $
+  $Id: ruli_util.h,v 1.4 2004/06/16 21:23:11 evertonm Exp $
   */
 
 
@@ -27,6 +27,9 @@ Boston, MA 02111-1307, USA.
 #define RULI_UTIL_H
 
 
+#include <sys/types.h>
+
+
 #ifndef RULI_MAX
 #define RULI_MAX(a, b) (((a) > (b)) ? (a) : (b))
 #endif
@@ -35,6 +38,14 @@ Boston, MA 02111-1307, USA.
 #define RULI_MIN(a, b) (((a) < (b)) ? (a) : (b))
 #endif
 
+#ifndef LONG_MAX
+#define LONG_MAX ( (long) ((((unsigned long) -1L) - 1) >> 1) )
+#endif
+
+#ifndef LONG_MIN
+#define LONG_MIN ( -(LONG_MAX + 1) )
+#endif
+
 typedef unsigned char  ruli_uint8_t;
 typedef unsigned short ruli_uint16_t;
 typedef unsigned int   ruli_uint32_t;
@@ -42,6 +53,6 @@ typedef unsigned int   ruli_uint32_t;
 ruli_uint8_t *ruli_unpack2(ruli_uint8_t *i, ruli_uint16_t val);
 ruli_uint16_t ruli_pack2(const ruli_uint8_t *i);
 ruli_uint32_t ruli_pack4(const ruli_uint8_t *i);
-
+void *ruli_memrchr(const void *buf, int c, size_t size);
 
 #endif /* RULI_UTIL_H */
