@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: addrsolver2.c,v 1.15 2004/06/03 17:34:27 evertonm Exp $
+  $Id: addrsolver2.c,v 1.16 2004/06/16 17:28:44 evertonm Exp $
  */
 
 
@@ -63,7 +63,7 @@ static void *clean_query(ruli_res_query_t *qry, char *domain)
 
 static void *on_answer(ruli_res_query_t *qry, void *arg)
 {
-  char       *domain  = arg;
+  char       *domain  = (char *) arg;
   const int  BUFSZ    = 1024;
   char       buf[BUFSZ];
   int        str_len;
@@ -129,7 +129,8 @@ static void *on_answer(ruli_res_query_t *qry, void *arg)
     ruli_parse_new(&parse);
 
     result = ruli_parse_message(&parse, &qry->answer_header, 
-				qry->answer_buf, qry->answer_buf_size);
+				(ruli_uint8_t *) qry->answer_buf,
+				qry->answer_buf_size);
     if (result) {
       str_len = snprintf(buf, BUFSZ, "%s answer-unparseable\n", domain);
       
@@ -146,7 +147,7 @@ static void *on_answer(ruli_res_query_t *qry, void *arg)
       ruli_addr_t addr;
       ruli_rr_t   *rr;
 
-      rr = ruli_list_get(&parse.answer_list, i);
+      rr = (ruli_rr_t *) ruli_list_get(&parse.answer_list, i);
 
       if (rr->qclass != RULI_RR_CLASS_IN)
 	continue;
@@ -183,10 +184,10 @@ static void *on_answer(ruli_res_query_t *qry, void *arg)
 	char      owner_buf[OWNER_BUFSZ];
 	int       owner_len;
 
-	result = ruli_dname_extract(qry->answer_buf, 
-				    qry->answer_buf + qry->answer_buf_size,
-				    owner_buf, 
-				    owner_buf + OWNER_BUFSZ,
+	result = ruli_dname_extract((ruli_uint8_t *) qry->answer_buf, 
+				    (ruli_uint8_t *) qry->answer_buf + qry->answer_buf_size,
+				    (ruli_uint8_t *) owner_buf, 
+				    (ruli_uint8_t *) owner_buf + OWNER_BUFSZ,
 				    rr->owner,
 				    &owner_len);
 	assert(!result);
@@ -253,14 +254,14 @@ static ruli_res_query_t *submit_query(ruli_res_t *res_ctx,
   /*
    * Allocate space for query
    */
-  qry = ruli_malloc(sizeof(ruli_res_query_t));
+  qry = (ruli_res_query_t *) ruli_malloc(sizeof(ruli_res_query_t));
   if (!qry)
     return 0;
   
   /*
    * Save domain string
    */
-  dom_str = ruli_malloc(domain_len + 1);
+  dom_str = (char *) ruli_malloc(domain_len + 1);
   if (!dom_str) {
     ruli_free(qry);
     return 0;
@@ -373,7 +374,7 @@ static void do_query(oop_source_sys *source_sys, oop_source *source,
   dname_buf_len = ruli_dname_encode_size(name, name_len);
 
   /* Allocate buffer */
-  dname_buf = ruli_malloc(dname_buf_len);
+  dname_buf = (char *) ruli_malloc(dname_buf_len);
   if (!dname_buf) {
     fprintf(stderr, 
 	    "%s: do_query(): ruli_malloc(%d) failed: %s\n",
@@ -525,7 +526,7 @@ static void parse_servers(ruli_list_t *server_list, int serverc,
 
 int main(int argc, const char **argv) 
 {
-  const char  *class = "in";
+  const char  *qclass = "in";
   const char  *type  = "a";
 
   int         retry;
@@ -565,9 +566,9 @@ int main(int argc, const char **argv)
 
   parse_servers(&server_list, serverc, serverv);
 
-  qc = ruli_get_qclass_code(class);
+  qc = ruli_get_qclass_code(qclass);
   if (!qc) {
-    fprintf(stderr, "%s: can't find query class: %s\n", prog_name, class);
+    fprintf(stderr, "%s: can't find query class: %s\n", prog_name, qclass);
     exit(1);
   }
 
