@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: hostsolver.c,v 1.17 2004/06/03 16:54:50 evertonm Exp $
+  $Id: hostsolver.c,v 1.18 2004/06/16 17:28:44 evertonm Exp $
  */
 
 
@@ -56,7 +56,7 @@ static void *on_stdout_write(oop_source *oop_src, int std_out,
 
   assert(event == OOP_WRITE);
   
-  str = ruli_list_pop(&output_list);
+  str = (char *) ruli_list_pop(&output_list);
 
   len = strlen(str);
 
@@ -82,7 +82,7 @@ static void enqueue_output(oop_source *oop_src, const char *str, int len,
 
   ++len;
 
-  buf = ruli_malloc(len);
+  buf = (char *) ruli_malloc(len);
   if (!buf) {
     fprintf(stderr, 
 	    "%s: enqueue_output(): ruli_malloc() failed\n", 
@@ -121,7 +121,7 @@ static void *on_answer(ruli_res_query_t *qry, void *arg)
 {
   oop_source *oop_src = qry->resolver->res_source;
   int        std_out  = 1;
-  char       *domain  = arg;
+  char       *domain  = (char *) arg;
   const int  BUFSZ    = 1024;
   char       buf[BUFSZ];
   int        str_len;
@@ -179,7 +179,8 @@ static void *on_answer(ruli_res_query_t *qry, void *arg)
     ruli_parse_new(&parse);
 
     result = ruli_parse_message(&parse, &qry->answer_header, 
-				qry->answer_buf, qry->answer_buf_size);
+				(ruli_uint8_t *) qry->answer_buf,
+				qry->answer_buf_size);
     if (result) {
       str_len = snprintf(buf, BUFSZ, "%s answer-unparseable\n", domain);
       
@@ -193,7 +194,7 @@ static void *on_answer(ruli_res_query_t *qry, void *arg)
       ruli_addr_t addr;
       ruli_rr_t   *rr;
 
-      rr = ruli_list_get(&parse.answer_list, i);
+      rr = (ruli_rr_t *) ruli_list_get(&parse.answer_list, i);
 
       if (rr->qclass != RULI_RR_CLASS_IN)
 	continue;
@@ -230,10 +231,10 @@ static void *on_answer(ruli_res_query_t *qry, void *arg)
 	char      owner_buf[OWNER_BUFSZ];
 	int       owner_len;
 
-	result = ruli_dname_extract(qry->answer_buf, 
-				    qry->answer_buf + qry->answer_buf_size,
-				    owner_buf, 
-				    owner_buf + OWNER_BUFSZ,
+	result = ruli_dname_extract((ruli_uint8_t *) qry->answer_buf, 
+				    (ruli_uint8_t *) qry->answer_buf + qry->answer_buf_size,
+				    (ruli_uint8_t *) owner_buf, 
+				    (ruli_uint8_t *) owner_buf + OWNER_BUFSZ,
 				    rr->owner,
 				    &owner_len);
 	assert(!result);
@@ -314,14 +315,14 @@ static ruli_res_query_t *submit_query(ruli_res_t *res_ctx,
   /*
    * Allocate space for query
    */
-  qry = ruli_malloc(sizeof(ruli_res_query_t));
+  qry = (ruli_res_query_t *) ruli_malloc(sizeof(ruli_res_query_t));
   if (!qry)
     return 0;
   
   /*
    * Save domain string
    */
-  dom_str = ruli_malloc(domain_len + 1);
+  dom_str = (char *) ruli_malloc(domain_len + 1);
   if (!dom_str) {
     ruli_free(qry);
     return 0;
@@ -368,8 +369,8 @@ static ruli_res_query_t *submit_query(ruli_res_t *res_ctx,
 static void *on_stdin_read(oop_source *oop_src, int std_in, 
 			   oop_event event, void *ctx)
 {
-  ruli_res_t  *res_ctx = ctx;
-  int         result;
+  ruli_res_t *res_ctx = (ruli_res_t *) ctx;
+  int        result;
 
   assert(std_in == 0);
   assert(event == OOP_READ);
@@ -448,7 +449,7 @@ static void *on_stdin_read(oop_source *oop_src, int std_in,
     dname_buf_len = ruli_dname_encode_size(domain, domain_len);
 
     /* Allocate buffer */
-    dname_buf = ruli_malloc(dname_buf_len);
+    dname_buf = (char *) ruli_malloc(dname_buf_len);
     if (!dname_buf) {
       fprintf(stderr, 
 	      "%s: on_stdin_read(): ruli_malloc(%d) failed: %s\n",
@@ -639,8 +640,8 @@ static void parse_servers(ruli_list_t *server_list, int serverc,
 
 int main(int argc, const char **argv) 
 {
-  const char  *class = "in";
-  const char  *type  = "a";
+  const char *qclass = "in";
+  const char *type  = "a";
 
   int         retry;
   int         timeout;
@@ -679,9 +680,9 @@ int main(int argc, const char **argv)
 
   parse_servers(&server_list, serverc, serverv);
 
-  qc = ruli_get_qclass_code(class);
+  qc = ruli_get_qclass_code(qclass);
   if (!qc) {
-    fprintf(stderr, "%s: can't find query class: %s\n", prog_name, class);
+    fprintf(stderr, "%s: can't find query class: %s\n", prog_name, qclass);
     exit(1);
   }
 
